/* 
Copyright (c) 2011, CyberData Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    Redistributions of source code must retain the above copyright notice,
	this list of conditions and the following disclaimer.
	
    Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
	
	Neither the name of CyberData Corporation nor the names of its contributors
	may be used to endorse or promote products derived from this software without
	specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/
#include <device.h>

#define DATA_SIZE 3									//Data transfer size
#define HISTORY_SIZE 4								//Size for the history buffer used to debouce the slider

void InitializeHardware();							//Initialize the hardware
void UpdateData();									//Read the CapSense sensor and update the data
void UpdateDisplay();								//Update the LCD display with the current data

static uint8 SliderPosition;						//The debounced position of the slider control
static uint8 Button0State;							//The state of button 0
static uint8 Button1State;							//The state of button 1
static int8 Joystick_Data[DATA_SIZE] = {0, 0, 0}; 	//Data buffer to be sent to the PC : [0] = X-Axis, [1] = Y-Axis, [2] = Buttons

void main()
{
    InitializeHardware();
   
    for(;;)
    {
		while(!USBFS_bGetEPAckState(1)); 		//Wait for ACK before loading data
        UpdateData();
		UpdateDisplay();
		
		Joystick_Data[0] = (int8)SliderPosition - 127;		//Center in range for PC joystick
		Joystick_Data[1] = (int8)SliderPosition - 127;		//Center in range for PC joystick
		Joystick_Data[2] = (Button0State ? 1 : 0) + (Button1State ? 2 : 0);
		
		USBFS_LoadInEP(1, Joystick_Data, DATA_SIZE);		//Place the data into the endpoint buffer
    }
}


void InitializeHardware()
{
	//Configure the LCD display, and display the initial message
	DebugLcd_Start();
	DebugLcd_ClearDisplay();
	DebugLcd_Position(0,4);
	DebugLcd_PrintString("SRS USB Demo");
	DebugLcd_Position(2,1);
	DebugLcd_PrintString("Connect to PC now.");
	
	//Turn on system interrupts
	CYGlobalIntEnable;
	
	//Configure the CapSense system
	CapSense_Start();
	CapSense_InitializeAllBaselines();
	CapSense_ScanEnabledWidgets();
		
	//Configure the USB system
	USBFS_Start(0, USBFS_3V_OPERATION);				//Start USBFS operation/device 0 and with 3V operation 
	while(!USBFS_bGetConfiguration());				//Wait for Device to enumerate 
    USBFS_LoadInEP(1, Joystick_Data, DATA_SIZE); 	//Loads an inital value into EP1 and sends it out to the PC	
}

void UpdateData()
{
static uint8 SliderHistory[HISTORY_SIZE];
uint8 CurrentSliderPosition; 
uint8 DataValid;
uint8 Counter;

	//Wait for the sensor to complete the current scan
	while(CapSense_IsBusy());
	
	//Read out the sensor data
	CapSense_UpdateEnabledBaselines();
	CapSense_ScanEnabledWidgets();
	Button0State = CapSense_CheckIsWidgetActive(CapSense_BUTTON0__BTN);
    Button1State = CapSense_CheckIsWidgetActive(CapSense_BUTTON1__BTN);
    CurrentSliderPosition = (uint8)CapSense_GetCentroidPos(CapSense_LINEARSLIDER0__LS);
	
	//Provide additional debounce when a finger is not touching the sensor (indicated by a value of 255)
	DataValid = 1;
	for(Counter = 0; Counter < HISTORY_SIZE; Counter ++)
	{
		if(SliderHistory[Counter] == 255)
		{
			DataValid = 0;
			break;
		}
	}
	if(DataValid)
	{
		SliderPosition = CurrentSliderPosition;
	}
	else
	{
		SliderPosition = 255;
	}
	//Update the debounce history for the slider control
	for(Counter = 1; Counter < HISTORY_SIZE; Counter++)
	{
		SliderHistory[Counter - 1] = SliderHistory[Counter];
	}
	SliderHistory[HISTORY_SIZE - 1] = CurrentSliderPosition;
}

void UpdateDisplay()
{
	DebugLcd_ClearDisplay();
	DebugLcd_Position(0,4);
	DebugLcd_PrintString("SRS USB Demo");
	DebugLcd_Position(1, 0);
	DebugLcd_PrintString("Button 0: ");
	DebugLcd_PrintString( Button0State ? "1" : "0");
	DebugLcd_Position(2, 0);
	DebugLcd_PrintString("Button 1: ");
	DebugLcd_PrintString( Button1State ? "1" : "0");
	DebugLcd_Position(3, 0);
	DebugLcd_PrintString("Slider:");
	if(SliderPosition < 255)
	{
		DebugLcd_DrawHorizontalBG(3, 8, 11, SliderPosition / 2);
	}	
}
/* [] END OF FILE */
